from datetime import datetime
from DateTime import DateTime
from ftw.casauth.testing import FTW_CASAUTH_INTEGRATION_TESTING
from ftw.testbrowser import browsing
from ftw.testing import freeze
from mock import patch
from plone import api
from plone.app.testing import TEST_USER_ID

import json
import unittest


class TestCASLogin(unittest.TestCase):

    layer = FTW_CASAUTH_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    @browsing
    def test_missing_ticket_returns_400(self, browser):
        with browser.expect_http_error(code=400, reason='Bad Request'):
            browser.open(
                self.portal.absolute_url() + '/@caslogin',
                method='POST',
                headers={'Accept': 'application/json'},
            )
        self.assertEqual(browser.status_code, 400)
        self.assertEqual(
            browser.json[u'error'][u'type'], u'Missing service ticket')

    @browsing
    def test_missing_cas_plugin_returns_501(self, browser):
        self.portal.acl_users._delOb('cas_auth')
        with browser.expect_http_error(code=501, reason='Not Implemented'):
            browser.open(
                self.portal.absolute_url() + '/@caslogin',
                data=json.dumps({
                    "ticket": "12345",
                }),
                method='POST',
                headers={'Accept': 'application/json',
                         'Content-Type': 'application/json'},
            )
        self.assertEqual(browser.status_code, 501)
        self.assertEqual(
            browser.json[u'error'][u'message'],
            u'CAS authentication plugin not installed.')

    @browsing
    def test_missing_jwt_plugin_returns_501(self, browser):
        self.portal.acl_users._delOb('jwt_auth')
        with browser.expect_http_error(code=501, reason='Not Implemented'):
            browser.open(
                self.portal.absolute_url() + '/@caslogin',
                data=json.dumps({
                    "ticket": "12345",
                }),
                method='POST',
                headers={'Accept': 'application/json',
                         'Content-Type': 'application/json'},
            )
        self.assertEqual(browser.status_code, 501)
        self.assertEqual(
            browser.json[u'error'][u'message'],
            u'JWT authentication plugin not installed.')

    @browsing
    def test_valid_ticket_returns_jwt_token(self, browser):
        with patch('ftw.casauth.restapi.caslogin.validate_ticket') as mock:
            mock.return_value = TEST_USER_ID
            browser.open(
                self.portal.absolute_url() + '/@caslogin',
                data=json.dumps({
                    "ticket": "12345",
                }),
                method='POST',
                headers={'Accept': 'application/json',
                         'Content-Type': 'application/json'},
            )
        self.assertEqual(browser.status_code, 200)
        self.assertIn(u'token', browser.json)

    @browsing
    def test_returns_ac_cookie_if_requested(self, browser):
        with patch('ftw.casauth.restapi.caslogin.validate_ticket') as mock:
            mock.return_value = TEST_USER_ID
            browser.open(
                self.portal.absolute_url() + '/@caslogin',
                data=json.dumps({
                    "ticket": "12345",
                    "set_cookie": True,
                }),
                method='POST',
                headers={'Accept': 'application/json',
                         'Content-Type': 'application/json'},
            )
        self.assertEqual(browser.status_code, 200)
        self.assertIn(u'userid', browser.json)
        self.assertIn(u'fullname', browser.json)
        self.assertIn(u'__ac', browser.cookies)

    @browsing
    def test_accepts_service_url_from_body(self, browser):
        with patch('ftw.casauth.restapi.caslogin.validate_ticket') as mock:
            mock.return_value = TEST_USER_ID
            browser.open(
                self.portal.absolute_url() + '/@caslogin',
                data=json.dumps({
                    "ticket": "12345",
                    "service": "http://myhost/#test",
                }),
                method='POST',
                headers={'Accept': 'application/json',
                         'Content-Type': 'application/json'},
            )
        self.assertEqual(browser.status_code, 200)
        self.assertIn(u'token', browser.json)
        mock.assert_called_with(
            u'12345', 'https://cas.domain.net', u'http://myhost/#test')

    @browsing
    def test_sets_login_times_when_success(self, browser):
        with patch('ftw.casauth.restapi.caslogin.validate_ticket') as mock:
            mock.return_value = TEST_USER_ID
            with freeze(datetime(2016, 2, 12, 16, 40)):
                browser.open(
                    self.portal.absolute_url() + '/@caslogin',
                    data=json.dumps({
                        "ticket": "12345",
                        "service": "http://myhost/#test",
                    }),
                    method='POST',
                    headers={'Accept': 'application/json',
                             'Content-Type': 'application/json'},
                )
                mtool = api.portal.get_tool('portal_membership')
                self.assertEqual(DateTime(),
                                 mtool.getMemberById(TEST_USER_ID).getProperty('login_time'))



from ftw.casauth.cas import strip_ticket
from ftw.casauth.cas import validate_ticket
from ftw.casauth.tests.utils import get_data
from ftw.casauth.tests.utils import MockResponse
from mock import patch

import unittest


class TestValdidateTicket(unittest.TestCase):

    @patch('ftw.casauth.cas.urllib2.urlopen')
    def test_validate_ticket_suceeds_with_valid_ticket(self, urlopen_mock):
        urlopen_mock.return_value = MockResponse(
            get_data('service_validate_success.xml'))
        self.assertEqual('james', validate_ticket(
            'ST-001-abc',
            'https://cas.domain.net',
            'https://service.domain.net'))

    @patch('ftw.casauth.cas.urllib2.urlopen')
    def test_validate_ticket_fails_with_invalid_ticket(self, urlopen_mock):
        urlopen_mock.return_value = MockResponse(
            get_data('service_validate_invalid_ticket.xml'))
        self.assertFalse(validate_ticket(
            'ST-001-abc',
            'https://cas.domain.net',
            'https://service.domain.net'))

    @patch('ftw.casauth.cas.urllib2.urlopen')
    def test_validate_ticket_fails_with_invalid_response(self, urlopen_mock):
        urlopen_mock.return_value = MockResponse("Invalid Response")
        self.assertFalse(validate_ticket(
            'ST-001-abc',
            'https://cas.domain.net',
            'https://service.domain.net'))

    @patch('ftw.casauth.cas.urllib2.urlopen')
    def test_validate_ticket_fails_with_invalid_xml_response(self, urlopen_mock):
        urlopen_mock.return_value = MockResponse("<resp>invalid</resp>")
        self.assertFalse(validate_ticket(
            'ST-001-abc',
            'https://cas.domain.net',
            'https://service.domain.net'))


class TestStripTicket(unittest.TestCase):

    def test_strip_ticket_drops_ticket_from_url(self):
        url = 'http://example.org?ticket=ST-001-abc&param1=v1&param2=v2'
        stripped = strip_ticket(url)
        self.assertEqual('http://example.org?param1=v1&param2=v2', stripped)

    def test_strip_ticket_preserves_multi_valued_params(self):
        url = 'http://example.org?ticket=ST-001-abc&multi=v1&multi=v2'
        stripped = strip_ticket(url)
        self.assertEqual('http://example.org?multi=v1&multi=v2', stripped)

    def test_strip_ticket_preserves_zope_style_multi_valued_params(self):
        url = 'http://example.org?ticket=ST-001-abc&multi:list=v1&multi:list=v2'
        stripped = strip_ticket(url)
        self.assertEqual('http://example.org?multi%3Alist=v1&multi%3Alist=v2', stripped)



from AccessControl.requestmethod import postonly
from AccessControl.SecurityInfo import ClassSecurityInfo
from DateTime import DateTime
from ftw.casauth.cas import service_url
from ftw.casauth.cas import validate_ticket
from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore.utils import getToolByName
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.PlonePAS.events import UserInitialLoginInEvent
from Products.PlonePAS.events import UserLoggedInEvent
from Products.PluggableAuthService.interfaces.plugins import IAuthenticationPlugin  # noqa
from Products.PluggableAuthService.interfaces.plugins import IChallengePlugin
from Products.PluggableAuthService.interfaces.plugins import IExtractionPlugin
from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from zope.component.hooks import getSite
from zope.event import notify
from zope.interface import implements
import urllib


manage_addCASAuthenticationPlugin = PageTemplateFile(
    "www/addPlugin", globals(), __name__="manage_addCASAuthenticationPlugin")


def addCASAuthenticationPlugin(self, id_, title=None, cas_server_url=None,
                               REQUEST=None):
    """Add a CAS authentication plugin
    """
    plugin = CASAuthenticationPlugin(id_, title, cas_server_url)
    self._setObject(plugin.getId(), plugin)

    if REQUEST is not None:
        REQUEST["RESPONSE"].redirect(
            "%s/manage_workspace"
            "?manage_tabs_message=CAS+authentication+plugin+added." %
            self.absolute_url()
        )


class CASAuthenticationPlugin(BasePlugin):
    """Plone PAS plugin for authentication against a CAS server.
    """
    implements(
        IAuthenticationPlugin,
        IChallengePlugin,
        IExtractionPlugin,
    )
    meta_type = "CAS Authentication Plugin"
    security = ClassSecurityInfo()

    # ZMI tab for configuration page
    manage_options = (
        ({'label': 'Configuration',
          'action': 'manage_config'},)
        + BasePlugin.manage_options
    )
    security.declareProtected(ManagePortal, 'manage_config')
    manage_config = PageTemplateFile('www/config', globals(),
                                     __name__='manage_config')

    def __init__(self, id_, title=None, cas_server_url=None):
        self._setId(id_)
        self.title = title
        if cas_server_url:
            cas_server_url = cas_server_url.rstrip('/')
        self.cas_server_url = cas_server_url

    security.declarePrivate('challenge')

    # Initiate a challenge to the user to provide credentials.
    def challenge(self, request, response, **kw):
        if 'ticket' in request.form:
            return False

        if not self.cas_server_url:
            return False

        response.redirect('%s/login?service=%s' % (
            self.cas_server_url,
            urllib.quote(service_url(request)),
        ), lock=True)
        return True

    security.declarePrivate('extractCredentials')

    # IExtractionPlugin implementation
    # Extracts a CAS service ticket from the request.
    def extractCredentials(self, request):
        if 'ticket' not in request.form:
            return None

        creds = {}
        creds['ticket'] = request.form.get('ticket')
        creds['service_url'] = service_url(request)

        # Avoid having the `ticket` query string param show up in the
        # user's browser's address bar by redirecting back to the
        # service_url, which should have the ticket stripped from it
        request.RESPONSE.redirect(creds['service_url'], lock=True)

        return creds

    security.declarePrivate('authenticateCredentials')

    # IAuthenticationPlugin implementation
    def authenticateCredentials(self, credentials):
        # Ignore credentials that are not from our extractor
        extractor = credentials.get('extractor')
        if extractor != self.getId():
            return None

        userid = validate_ticket(
            credentials['ticket'],
            self.cas_server_url,
            credentials['service_url'],
        )
        if not userid:
            return None

        result = self.login_user(userid)
        if not result:
            return None

        return userid, userid

    def handle_login(self, userid):
        pas = self._getPAS()
        info = pas._verifyUser(pas.plugins, user_id=userid)
        if info is None:
            return None

        mtool = getToolByName(getSite(), 'portal_membership')
        member = mtool.getMemberById(userid)
        if member is None:
            return None

        first_login = self.set_login_times(member)
        self.fire_login_events(first_login, member)
        self.expire_clipboard()
        mtool.createMemberArea(member_id=userid)
        return member

    def login_user(self, userid):
        member = self.handle_login(userid)
        if not member:
            return None
        pas = self._getPAS()
        pas.updateCredentials(
            self.REQUEST, self.REQUEST.RESPONSE, member.getUserName(), '')
        return member

    def set_login_times(self, member):
        # The return value indicates if this is the first logged login time.

        first_login = False
        default = DateTime('2000/01/01')

        login_time = member.getProperty('login_time', default)
        if login_time == default:
            first_login = True
            login_time = DateTime()

        mtool = getToolByName(getSite(), 'portal_membership')
        member.setMemberProperties(dict(
            login_time=mtool.ZopeTime(),
            last_login_time=login_time))

        return first_login

    def fire_login_events(self, first_login, user):
        if first_login:
            notify(UserInitialLoginInEvent(user))
        else:
            notify(UserLoggedInEvent(user))

    def expire_clipboard(self):
        if self.REQUEST.get('__cp', None) is not None:
            self.REQUEST.RESPONSE.expireCookie('__cp', path='/')

    security.declareProtected(ManagePortal, 'manage_updateConfig')

    @postonly
    def manage_updateConfig(self, REQUEST):
        """Update configuration of CAS Authentication Plugin.
        """
        response = REQUEST.response

        self.cas_server_url = REQUEST.form.get('cas_server_url', '').rstrip('/')

        response.redirect('%s/manage_config?manage_tabs_message=%s' %
                          (self.absolute_url(), 'Configuration+updated.'))



from collections import OrderedDict
from DateTime import DateTime
from datetime import datetime
from ftw.casauth.cas import service_url
from ftw.casauth.testing import FTW_CASAUTH_INTEGRATION_TESTING
from ftw.testing import freeze
from mock import patch
from plone.app.testing import TEST_USER_ID
from Products.CMFCore.utils import getToolByName
from urllib import urlencode
from zope.component.hooks import getSite
import unittest


class TestCASAuthPlugin(unittest.TestCase):

    layer = FTW_CASAUTH_INTEGRATION_TESTING

    def setUp(self):
        self.plugin = self.layer['plugin']
        self.request = self.layer['request']

    def test_challenge_redirects_to_cas(self):
        response = self.request.response
        self.assertTrue(self.plugin.challenge(self.request, response))

        self.assertEqual(302, response.getStatus())
        self.assertEqual(
            'https://cas.domain.net/login?service=http%3A//nohost',
            response.getHeader('Location'))

    def test_challenge_doesnt_redirect_with_ticket(self):
        self.request.form.update(dict(ticket='ST-001-abc'))
        response = self.request.response
        self.assertFalse(self.plugin.challenge(self.request, response))

        self.assertEqual(200, response.getStatus())

    def test_challenge_redirect_includes_query_string(self):
        self.request.environ['QUERY_STRING'] = 'param1=value1&param2=value2'
        response = self.request.response

        self.assertTrue(self.plugin.challenge(self.request, response))

        self.assertEqual(302, response.getStatus())
        self.assertEqual(
            'https://cas.domain.net/login?service=http%3A//nohost%3Fparam1%3Dvalue1%26param2%3Dvalue2',
            response.getHeader('Location'))

    def test_challenge_doesnt_redirect_with_missing_cas_server_url(self):
        self.plugin.cas_server_url = None
        response = self.request.response
        self.assertFalse(self.plugin.challenge(self.request, response))

    def test_extract_credentials_returns_ticket(self):
        self.request.form.update(dict(ticket='ST-001-abc'))
        creds = self.plugin.extractCredentials(self.request)

        self.assertIn('ticket', creds)
        self.assertEqual('ST-001-abc', creds['ticket'])

    def test_extract_credentials_returns_service_url(self):
        self.request.form.update({'ticket': 'ST-001-abc'})
        creds = self.plugin.extractCredentials(self.request)

        self.assertIn('service_url', creds)
        self.assertEqual('http://nohost', creds['service_url'])

    def test_extract_credentials_redirects_to_service_url(self):
        self.request.form.update({'ticket': 'ST-001-abc'})
        self.plugin.extractCredentials(self.request)

        self.assertEqual(302, self.request.RESPONSE.status)
        self.assertDictContainsSubset({'location': 'http://nohost'},
                                      self.request.RESPONSE.headers)

    def test_extract_credentials_without_ticket_returns_none(self):
        creds = self.plugin.extractCredentials(self.request)

        self.assertEqual(None, creds)

    def test_extract_credentials_without_ticket_doesnt_redirect(self):
        self.plugin.extractCredentials(self.request)

        self.assertEqual(200, self.request.RESPONSE.status)
        self.assertNotIn('location', self.request.RESPONSE.headers)

    def test_service_url_has_ticket_stripped(self):
        params = OrderedDict(
            (('ticket', 'ST-001-abc'), ('param1', 'v1'), ('param2', 'v2')))
        self.request.form.update(params)
        self.request.environ['QUERY_STRING'] = urlencode(params)
        url = service_url(self.request)

        self.assertEqual('http://nohost?param1=v1&param2=v2', url)

    @patch('ftw.casauth.plugin.validate_ticket')
    def test_authenticate_credentials_succeeds_with_valid_credentials(self, mock_validate_ticket):
        mock_validate_ticket.return_value = TEST_USER_ID
        creds = {
            'extractor': self.plugin.getId(),
            'ticket': 'ST-001-abc',
            'service_url': 'http://127.0.0.1/'
        }
        self.plugin.REQUEST = self.request
        self.plugin.REQUEST.RESPONSE = self.request.response
        userid, login = self.plugin.authenticateCredentials(creds)
        self.assertEqual(TEST_USER_ID, userid)
        self.assertEqual(TEST_USER_ID, login)

    def test_authenticate_credentials_fails_with_wrong_extractor(self):
        creds = {
            'extractor': 'another-plugin',
            'ticket': 'ST-001-abc',
            'service_url': 'http://127.0.0.1/'
        }
        ret = self.plugin.authenticateCredentials(creds)
        self.assertEqual(None, ret)

    @patch('ftw.casauth.plugin.validate_ticket')
    def test_authenticate_credentials_fails_with_inexisting_user(self, mock_validate_ticket):
        mock_validate_ticket.return_value = 'james'
        creds = {
            'extractor': self.plugin.getId(),
            'ticket': 'ST-001-abc',
            'service_url': 'http://127.0.0.1/'
        }
        self.plugin.REQUEST = self.request
        self.plugin.REQUEST.RESPONSE = self.request.response
        ret = self.plugin.authenticateCredentials(creds)
        self.assertEqual(None, ret)

    @patch('ftw.casauth.plugin.validate_ticket')
    def test_sets_login_times_when_success(self, mock_validate_ticket):
        mtool = getToolByName(getSite(), 'portal_membership')

        mock_validate_ticket.return_value = TEST_USER_ID
        creds = {
            'extractor': self.plugin.getId(),
            'ticket': 'ST-001-abc',
            'service_url': 'http://127.0.0.1/'
        }
        self.plugin.REQUEST = self.request
        self.plugin.REQUEST.RESPONSE = self.request.response

        with freeze(datetime(2016, 2, 12, 16, 40)):
            userid, login = self.plugin.authenticateCredentials(creds)
            self.assertEqual(
                DateTime(),
                mtool.getMemberById(TEST_USER_ID).getProperty('login_time'))

    @patch('ftw.casauth.plugin.validate_ticket')
    def test_expires_clipboard_when_success(self, mock_validate_ticket):
        mock_validate_ticket.return_value = TEST_USER_ID
        creds = {
            'extractor': self.plugin.getId(),
            'ticket': 'ST-001-abc',
            'service_url': 'http://127.0.0.1/'
        }
        self.plugin.REQUEST = self.request
        self.plugin.REQUEST.RESPONSE = self.request.response
        self.plugin.REQUEST.RESPONSE.setCookie('__cp', ['file-1'], path='/')
        self.plugin.REQUEST['__cp'] = ['file-1']

        userid, login = self.plugin.authenticateCredentials(creds)

        cookie = self.plugin.REQUEST.RESPONSE.cookies.get('__cp')
        self.assertEqual('Wed, 31-Dec-97 23:59:59 GMT', cookie.get('expires'))
        self.assertEqual('deleted', cookie.get('value'))



from setuptools import setup, find_packages
import os

version = '1.4.1'

tests_require = [
    'plone.api',
    'plone.app.testing',
    'plone.restapi',
    'mock',
    'ftw.testbrowser',
    'ftw.testing',
]

install_requires = [
    'Plone',
    'setuptools',
]


setup(
    name='ftw.casauth',
    version=version,
    description='Plone PAS plugin for authentication against CAS.',
    long_description=(open('README.rst').read() + '\n' +
                      open(os.path.join('docs', 'HISTORY.txt')).read()),
    classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.3',
        'Framework :: Plone :: 5.1',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    keywords='',
    author='Thomas Buchberger',
    author_email='mailto:t.buchberger@4teamwork.ch',
    url='https://github.com/4teamwork/ftw.casauth',
    license='GPL2',

    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['ftw'],
    include_package_data=True,
    zip_safe=False,

    install_requires=install_requires,
    tests_require=tests_require,
    extras_require=dict(tests=tests_require),
    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-
from ftw.casauth.cas import service_url
from ftw.casauth.cas import validate_ticket
from plone.restapi.deserializer import json_body
from plone.restapi.services import Service
from Products.CMFCore.utils import getToolByName
from Products.PluggableAuthService.interfaces.plugins import IAuthenticationPlugin  # noqa
from zope.interface import alsoProvides

import plone.protect.interfaces


class CASLogin(Service):
    """Handles login and returns a JSON web token (JWT).
    """
    def reply(self):
        data = json_body(self.request)
        if 'ticket' not in data:
            self.request.response.setStatus(400)
            return dict(error=dict(
                type='Missing service ticket',
                message='Service ticket must be provided in body.'))

        if 'service' in data:
            service = data['service']
        else:
            service = service_url(self.request)[:-10],  # Strip `/@caslogin`

        set_cookie = data.get('set_cookie', False)

        # Disable CSRF protection
        if 'IDisableCSRFProtection' in dir(plone.protect.interfaces):
            alsoProvides(self.request,
                         plone.protect.interfaces.IDisableCSRFProtection)

        uf = getToolByName(self.context, 'acl_users')
        plugins = uf._getOb('plugins')
        authenticators = plugins.listPlugins(IAuthenticationPlugin)
        cas_plugin = None
        jwt_plugin = None
        for id_, authenticator in authenticators:
            if authenticator.meta_type == "CAS Authentication Plugin":
                cas_plugin = authenticator
            elif authenticator.meta_type == "JWT Authentication Plugin":
                jwt_plugin = authenticator

        if cas_plugin is None:
            self.request.response.setStatus(501)
            return dict(error=dict(
                type='Login failed',
                message='CAS authentication plugin not installed.'))

        if jwt_plugin is None and not set_cookie:
            self.request.response.setStatus(501)
            return dict(error=dict(
                type='Login failed',
                message='JWT authentication plugin not installed.'))

        userid = validate_ticket(
            data['ticket'],
            cas_plugin.cas_server_url,
            service,
        )

        user = uf.getUserById(userid) if userid else None
        if not user:
            return dict(error=dict(
                type='Login failed',
                message='User with userid {} not found.'.format(userid)))

        if set_cookie:
            cas_plugin.login_user(userid)
            return {'userid': userid, 'fullname': user.getProperty('fullname')}
        else:
            cas_plugin.handle_login(userid)
            payload = {'fullname': user.getProperty('fullname')}
            return {
                'token': jwt_plugin.create_token(userid, data=payload)
            }

    def check_permission(self):
        return



from logging import getLogger
from urllib import urlencode
from urlparse import parse_qsl
from urlparse import urlsplit
from urlparse import urlunsplit
from xml.dom.minidom import parseString
from xml.parsers.expat import ExpatError

import urllib
import urllib2
import ssl


CAS_NS = "http://www.yale.edu/tp/cas"

logger = getLogger('ftw.casauth')


def validate_ticket(ticket, cas_server_url, service_url):
    """Validates a CAS service ticket and returns the authenticated userid.
    """
    validate_url = '%s/serviceValidate?service=%s&ticket=%s' % (
        cas_server_url,
        urllib.quote(service_url),
        ticket,
    )

    try:
        resp = urllib2.urlopen(validate_url)
    except urllib2.HTTPError as e:
        logger.warning("Ticket validation failed. Could not open url %s. "
                       "Staus code: %s, reason: %s", validate_url, e.code,
                       e.reason)
        return False
    except urllib2.URLError as e:
        logger.warning("Ticket validation failed. Could not open url %s. "
                       "Reason: %s", validate_url, e.reason)
        return False
    except ssl.CertificateError as e:
        logger.warning("Ticket validation failed. Could not open url %s. "
                       "CertificateError: %s", validate_url, str(e))
        return False
    except ssl.SSLError as e:
        logger.warning("Ticket validation failed. Could not open url %s. "
                       "SSL Error: %s", validate_url, e.reason)
        return False

    try:
        resp_data = resp.read()
    except IOError:
        logger.warning("Ticket validation failed. Could not read from url %s.",
                       validate_url)
        return False

    try:
        doc = parseString(resp_data)
    except ExpatError:
        return False
    auth_success = doc.getElementsByTagNameNS(CAS_NS,
                                              'authenticationSuccess')
    if not auth_success:
        auth_fail = doc.getElementsByTagNameNS(CAS_NS,
                                               'authenticationFailure')
        if auth_fail:
            logger.info(
                "Authentication failed: Service ticket validation returned"
                " '%s'." % auth_fail[0].getAttribute('code'))
        else:
            logger.info("Authentication failed: Could not validate service"
                        " ticket.")
        return False

    userid = auth_success[0].getElementsByTagNameNS(CAS_NS, 'user')
    if not userid:
        return False
    userid = userid[0].firstChild.data

    return userid


def service_url(request):
    url = request['ACTUAL_URL']
    if request['QUERY_STRING']:
        url = '%s?%s' % (url, request['QUERY_STRING'])
        url = strip_ticket(url)
    return url


def strip_ticket(url):
    """Drop the `ticket` query string parameter from a given URL,
    but preserve everything else.
    """
    scheme, netloc, path, query, fragment = urlsplit(url)
    # Using parse_qsl here to preserve order
    qs_params = filter(lambda (k, v): k != 'ticket', parse_qsl(query))
    query = urlencode(qs_params)
    new_url = urlunsplit((scheme, netloc, path, query, fragment))
    return new_url
